import sys
import os
import tempfile
import shutil
import unittest
import subprocess

import sandglass

__all__ = [ "TestCaseTempDir", "TestCaseTree" ]
#__all__ = [ "TestCaseTempDir", "TestCaseTree", "TestCaseCLI", "CLIToolFailureException" ]

FAUX_DIRECTORY_SKEL = [
		"etc/passwd",
		"etc/shadow",
		"etc/hosts",
		"etc/fedora-release",
		"etc/sysctl.conf",
		"etc/sysconfig/init",
		"etc/sysconfig/selinux",
		"etc/not.owned",
		"opt/foo/etc/bar.conf",
]

class FauxDirectoryTree(object):
	"""Faux directory tree used for testing."""

	def __init__(self):
		self._path = tempfile.mkdtemp()
		self._populate()

	@property
	def path(self):
		return self._path

	def clean(self):
		if self._path is None: return
		shutil.rmtree(self._path, True)

	def _populate(self):
		for f in FAUX_DIRECTORY_SKEL:
			self._create_file(f)

	def _create_file(self, name):
		fullname = os.path.join(self._path, name)
		dir = os.path.dirname(fullname)
		if not os.path.exists(dir):
			os.makedirs(dir)
		with open(fullname, "a+") as fd:
			fd.write("sample content for %s" % name)

class TestCaseTempDir(unittest.TestCase):
	"""Base class for all unit tests, which need temporary directory."""

	def setUp(self):
		self.temp_dir = tempfile.mkdtemp()

	def tearDown(self):
		shutil.rmtree(self.temp_dir, True)

class TestCaseTree(TestCaseTempDir):
	"""Base class for all Sandglass unit tests, which work with directory tree."""

	def setUp(self):
		TestCaseTempDir.setUp(self)
		self.tree = FauxDirectoryTree()
		self.sgl = sandglass.Sandglass(data_dir=self.temp_dir, root_dir=self.tree.path)
		self.assertIsInstance(self.sgl, sandglass.Sandglass)

	def tearDown(self):
		TestCaseTempDir.tearDown(self)
		self.tree.clean()

#class TestCaseCLI(TestCaseTree):
#	"""Base class for Sandglass CLI tests."""
#
#	def _run_internal(self, args):
#		exit_code = 0
#		try:
#			result = sglcli.parse_args(args)
#			result.command(result, self.sgl)
#		except SystemExit as e:
#			exit_code = e.code
#
#		return exit_code
#
#	def _run(self, args):
#		"""
#		Simulates the CLI command execution. If the tool exits with non-zero error code,
#		or any Sandglass exception is catched, CLIToolFailureException is raised.
#
#		- stdout and stderr are temporarily redirected to /dev/null
#		- the execution is performed and sys.exit is catched
#		- other errors are reraised
#		"""
#		raised = None
#		try:
#			old_stdout, old_stderr = sys.stdout, sys.stderr
#			devnull = open("/dev/null", "w")
#			sys.stdout = sys.stderr = devnull
#			exit_code = self._run_internal(args)
#		except Exception as e:
#			raised = e
#		finally:
#			sys.stdout, sys.stderr = old_stdout, old_stderr
#
#		if raised is not None:
#			if isinstance(raised, sandglass.exceptions.SandglassException):
#				raise CLIToolFailureException(str(raised))
#			else:
#				raise raised
#		elif exit_code != 0:
#			raise CLIToolFailureException("Tool execution unsuccessful (exit code %d)." % exit_code)
#
#class CLIToolFailureException(Exception):
#	def __init__(self, message):
#		self._message = message
#
#	def __str__(self):
#		return self._message
